/*
 * Name:    d_wavenumber.h 
 *
 * Purpose: Header file for MX motor driver to control an MX motor device
 *          in units of X-ray wavenumber.
 *
 * Author:  William Lavender
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 1999, 2001 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __D_WAVENUMBER_H__
#define __D_WAVENUMBER_H__

#include "mx_motor.h"
#include "mx_constants.h"

/* ===== MX wavenumber motor data structures ===== */

typedef struct {
	MX_RECORD *dependent_motor_record;
	MX_RECORD *d_spacing_record;
	double angle_scale; /* To translate betw. radians and native units. */
} MX_WAVENUMBER_MOTOR;

/* Define all of the interface functions. */

MX_API mx_status_type mxd_wavenum_motor_initialize_type( long type );
MX_API mx_status_type mxd_wavenum_motor_create_record_structures(
					MX_RECORD *record );
MX_API mx_status_type mxd_wavenum_motor_finish_record_initialization(
					MX_RECORD *record );
MX_API mx_status_type mxd_wavenum_motor_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_wavenum_motor_print_structure(
					FILE *file, MX_RECORD *record );
MX_API mx_status_type mxd_wavenum_motor_read_parms_from_hardware(
						MX_RECORD *record );
MX_API mx_status_type mxd_wavenum_motor_write_parms_to_hardware(
						MX_RECORD *record );
MX_API mx_status_type mxd_wavenum_motor_open( MX_RECORD *record );
MX_API mx_status_type mxd_wavenum_motor_close( MX_RECORD *record );
MX_API mx_status_type mxd_wavenum_motor_motor_is_busy( MX_MOTOR *motor );
MX_API mx_status_type mxd_wavenum_motor_move_absolute( MX_MOTOR *motor );
MX_API mx_status_type mxd_wavenum_motor_get_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_wavenum_motor_set_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_wavenum_motor_soft_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_wavenum_motor_immediate_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_wavenum_motor_positive_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_wavenum_motor_negative_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_wavenum_motor_find_home_position( MX_MOTOR *motor );

extern MX_RECORD_FUNCTION_LIST mxd_wavenum_motor_record_function_list;
extern MX_MOTOR_FUNCTION_LIST mxd_wavenum_motor_motor_function_list;

extern long mxd_wavenum_motor_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_wavenum_motor_rfield_def_ptr;

#define MXD_WAVENUM_MOTOR_STANDARD_FIELDS \
  {-1, -1, "dependent_motor_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, \
		offsetof(MX_WAVENUMBER_MOTOR, dependent_motor_record ), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY) }, \
  \
  {-1, -1, "d_spacing_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_WAVENUMBER_MOTOR, d_spacing_record),\
	{0}, NULL, MXFF_IN_DESCRIPTION}, \
  \
  {-1, -1, "angle_scale", MXFT_DOUBLE, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_WAVENUMBER_MOTOR, angle_scale),\
	{0}, NULL, MXFF_IN_DESCRIPTION}

#endif /* __D_WAVENUMBER_H__ */
