/*
 * Name:    mx_driver.c
 *
 * Purpose: Describes the list of interfaces and devices included in
 *          this version of the program.
 *
 * Author:  William Lavender
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 1999-2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>

#include "mxconfig.h"
#include "mx_osdef.h"
#include "mx_types.h"

#include "mx_driver.h"

/* -- Include header files that define MX_XXX_FUNCTION_LIST structures. -- */

#include "mx_rs232.h"
#include "mx_gpib.h"
#include "mx_camac.h"
#include "mx_generic.h"
#include "mx_portio.h"
#include "mx_vme.h"

#if USE_TCPIP

#include "mx_socket.h"

#endif /* USE_TCPIP */

#include "mx_analog_input.h"
#include "mx_analog_output.h"
#include "mx_autoscale.h"
#include "mx_digital_input.h"
#include "mx_digital_output.h"
#include "mx_motor.h"
#include "mx_encoder.h"
#include "mx_scaler.h"
#include "mx_timer.h"
#include "mx_amplifier.h"
#include "mx_relay.h"
#include "mx_mca.h"
#include "mx_mce.h"
#include "mx_mcs.h"
#include "mx_table.h"

#include "mx_scan.h"
#include "mx_scan_linear.h"
#include "mx_scan_list.h"
#include "mx_scan_xafs.h"
#include "mx_scan_quick.h"

#include "mx_variable.h"
#include "mx_vinline.h"
#include "mx_vnet.h"
#include "mx_vepics.h"

#include "mx_net.h"

#include "mx_list_head.h"

#include "mx_dead_reckoning.h"

/* Include the header files for all of the interfaces and devices. */

#include "i_soft_rs232.h"
#include "i_network_rs232.h"
#include "i_tty.h"
#include "i_dos_com.h"
#include "i_fossil.h"
#include "i_ks3344.h"

#ifdef OS_WIN32
#include "i_win32_com.h"
#endif

#if USE_TCPIP
#include "i_tcp232.h"
#endif

#include "i_ni488.h"
#include "i_k500serial.h"
#include "i_soft_camac.h"
#include "i_dsp6001.h"
#include "i_pdi45.h"
#include "i_pdi40.h"
#include "i_8255.h"
#include "i_6821.h"
#include "i_newport.h"
#include "i_pmac.h"
#include "i_compumotor.h"
#include "i_ggcs.h"
#include "i_d8.h"
#include "i_vp9000.h"
#include "i_ortec974.h"
#include "i_am9513.h"
#include "i_hsc1.h"
#include "i_pcmotion32.h"
#include "i_pcstep.h"
#include "i_linux_portio.h"
#include "i_dos_portio.h"
#include "i_driverlinx_portio.h"
#include "i_databox.h"
#include "i_vxi_memacc.h"

#include "d_ks3512.h"
#include "d_ks3112.h"
#include "d_ks3063.h"
#include "d_8255.h"
#include "d_6821.h"
#include "d_bit.h"
#include "d_soft_dinput.h"
#include "d_soft_doutput.h"
#include "d_network_ainput.h"
#include "d_network_aoutput.h"
#include "d_network_dinput.h"
#include "d_network_doutput.h"
#include "d_vme_dio.h"
#include "d_pmac_aio.h"
#include "d_pmac_dio.h"
#include "d_compumotor_dio.h"
#include "d_mca_value.h"

#include "d_soft_motor.h"
#include "d_e500.h"
#include "d_smc24.h"
#include "d_panther.h"
#include "d_mmc32.h"
#include "d_pm304.h"
#include "d_newport.h"
#include "d_pmac.h"
#include "d_pmactc.h"
#include "d_aps_gap.h"
#include "d_compumotor.h"
#include "d_network_motor.h"
#include "d_ggcs.h"
#include "d_d8.h"
#include "d_vp9000.h"
#include "d_e662.h"
#include "d_hsc1.h"
#include "d_stp100.h"
#include "d_pcmotion32.h"
#include "d_pcstep.h"
#include "d_dac_motor.h"
#include "d_lakeshore330_motor.h"
#include "d_disabled_motor.h"

#include "d_energy.h"
#include "d_wavelength.h"
#include "d_wavenumber.h"
#include "d_slit_motor.h"
#include "d_trans_motor.h"
#include "d_xafs_wavenumber.h"
#include "d_delta_motor.h"
#include "d_elapsed_time.h"
#include "d_monochromator.h"
#include "d_linear_function.h"
#include "d_table_motor.h"
#include "d_theta_2theta.h"
#include "d_qmotor.h"
#include "d_segmented_move.h"
#include "d_tangent_arm.h"
#include "d_aframe_detector_motor.h"

#include "d_aps_10id.h"
#include "d_aps_18id.h"
#include "d_compumotor_linear.h"
#include "d_compumotor_trans.h"

#include "d_ks3640.h"
#include "d_soft_scaler.h"
#include "d_network_scaler.h"
#include "d_auto_scaler.h"
#include "d_gain_tracking_scaler.h"
#include "d_scaler_function.h"
#include "d_qs450.h"
#include "d_ortec974_scaler.h"
#include "d_ortec974_timer.h"
#include "d_soft_timer.h"
#include "d_network_timer.h"
#include "d_timer_fanout.h"
#include "d_rtc018.h"
#include "d_generic_relay.h"
#include "d_blind_relay.h"
#include "d_soft_amplifier.h"
#include "d_net_amplifier.h"
#include "d_keithley428.h"
#include "d_adc_table.h"
#include "d_pdi45.h"
#include "d_pdi40.h"
#include "d_labpc.h"
#include "d_am9513_motor.h"
#include "d_am9513_scaler.h"
#include "d_am9513_timer.h"
#include "d_soft_mca.h"
#include "d_network_mca.h"
#include "d_sis3801.h"

#include "d_mca_channel.h"
#include "d_mca_roi_integral.h"
#include "d_mca_timer.h"
#include "d_mca_alt_time.h"

#include "d_databox_motor.h"
#include "d_databox_scaler.h"
#include "d_databox_timer.h"
#include "d_databox_mcs.h"
#include "d_databox_mce.h"

#include "s_input.h"
#include "s_motor.h"
#include "s_theta_2theta.h"
#include "s_slit.h"
#include "sl_file.h"
#include "sxafs_std.h"
#include "sq_mcs.h"
#include "sq_joerger.h"
#include "sq_aps_id.h"

#include "d_auto_amplifier.h"
#include "d_auto_filter.h"
#include "d_auto_filter_amplifier.h"
#include "d_auto_net.h"

#include "d_soft_mcs.h"
#include "d_network_mcs.h"
#include "d_mcs_scaler.h"
#include "d_mcs_timer.h"
#include "d_mcs_mce.h"

#include "d_network_mce.h"

#include "i_scipe.h"
#include "d_scipe_motor.h"
#include "d_scipe_scaler.h"
#include "d_scipe_timer.h"

#include "v_mathop.h"
#include "v_old_mathop.h"

#if USE_TCPIP

#include "n_tcpip.h"

#endif /* USE_TCPIP */

#if HAVE_EPICS
#include "i_epics_rs232.h"
#include "i_epics_gpib.h"
#include "d_epics_mca.h"
#include "d_epics_mcs.h"
#include "d_epics_motor.h"
#include "d_epics_scaler.h"
#include "d_epics_timer.h"
#include "v_aps_topup.h"
#endif

#if HAVE_LINUX_VME58
#include "i_vme58.h"
#include "d_vme58.h"
#endif

#if HAVE_ORTEC_UMCBI
#include "i_umcbi.h"
#include "d_trump.h"
#endif

#if USE_TCPIP
#include "i_xia_mds.h"
#include "i_xia_network.h"
#endif
#if HAVE_XIA_XERXES
#include "i_xia_xerxes.h"
#endif

#if USE_TCPIP || HAVE_XIA_XERXES
#include "d_xia_dxp_mca.h"
#include "d_xia_dxp_var.h"
#include "d_xia_dxp_sum.h"
#endif

/* -- Define the list of record types. -- */

MX_DRIVER *mx_list_of_types[] = {
	mx_superclass_list,
	mx_class_list,
	mx_type_list,
	NULL
};

/* -- Define lists that relate types to classes and to function lists. -- */

  /****************** Record Superclasses ********************/

MX_DRIVER mx_superclass_list[] = {
{"list_head_sclass", 0, 0, MXR_LIST_HEAD,     NULL, NULL, NULL, NULL, NULL},
{"interface",        0, 0, MXR_INTERFACE,     NULL, NULL, NULL, NULL, NULL},
{"device",           0, 0, MXR_DEVICE,        NULL, NULL, NULL, NULL, NULL},
{"scan",             0, 0, MXR_SCAN,          NULL, NULL, NULL, NULL, NULL},
{"variable",         0, 0, MXR_VARIABLE,      NULL, NULL, NULL, NULL, NULL},
{"server",           0, 0, MXR_SERVER,        NULL, NULL, NULL, NULL, NULL},

{"", 0, 0, 0, NULL, NULL, NULL, NULL, NULL}
};

  /********************* Record Classes **********************/

MX_DRIVER mx_class_list[] = {

{"list_head_class", 0, MXL_LIST_HEAD,      MXR_LIST_HEAD,
				NULL, NULL, NULL, NULL, NULL},

  /* ================== Interface classes ================== */

{"rs232",          0, MXI_RS232,          MXR_INTERFACE,
				NULL, NULL, NULL, NULL, NULL},
{"gpib",           0, MXI_GPIB,           MXR_INTERFACE,
				NULL, NULL, NULL, NULL, NULL},
{"camac",          0, MXI_CAMAC,          MXR_INTERFACE,
				NULL, NULL, NULL, NULL, NULL},
{"generic",        0, MXI_GENERIC,        MXR_INTERFACE,
				NULL, NULL, NULL, NULL, NULL},
{"portio",         0, MXI_PORTIO,         MXR_INTERFACE,
				NULL, NULL, NULL, NULL, NULL},
{"vme",            0, MXI_VME,            MXR_INTERFACE,
				NULL, NULL, NULL, NULL, NULL},

  /* =================== Device classes =================== */

{"analog_input",   0, MXC_ANALOG_INPUT,   MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"analog_output",  0, MXC_ANALOG_OUTPUT,  MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"digital_input",  0, MXC_DIGITAL_INPUT,  MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"digital_output", 0, MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"motor",          0, MXC_MOTOR,          MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"encoder",        0, MXC_ENCODER,        MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"scaler",         0, MXC_SCALER,         MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"timer",          0, MXC_TIMER,          MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"amplifier",      0, MXC_AMPLIFIER,      MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"relay",          0, MXC_RELAY,          MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"mca",            0, MXC_MULTICHANNEL_ANALYZER, MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"mce",            0, MXC_MULTICHANNEL_ENCODER, MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"mcs",            0, MXC_MULTICHANNEL_SCALER, MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"table",          0, MXC_TABLE,          MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"autoscale",      0, MXC_AUTOSCALE,      MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},

  /* ==================== Scan classes ==================== */

{"linear_scan",    0, MXS_LINEAR_SCAN,    MXR_SCAN,
				NULL, NULL, NULL, NULL, NULL},
{"list_scan",      0, MXS_LIST_SCAN,      MXR_SCAN,
				NULL, NULL, NULL, NULL, NULL},
{"xafs_scan_class",0, MXS_XAFS_SCAN,      MXR_SCAN,
				NULL, NULL, NULL, NULL, NULL},
{"quick_scan",     0, MXS_QUICK_SCAN,     MXR_SCAN,
				NULL, NULL, NULL, NULL, NULL},

  /* ================== Variable classes ================== */

{"inline",         0, MXV_INLINE,         MXR_VARIABLE,
				NULL, NULL, NULL, NULL, NULL},
{"net_variable",   0, MXV_NETWORK,        MXR_VARIABLE,
				NULL, NULL, NULL, NULL, NULL},
{"epics_variable", 0, MXV_EPICS,          MXR_VARIABLE,
				NULL, NULL, NULL, NULL, NULL},
{"calc",           0, MXV_CALC,           MXR_VARIABLE,
				NULL, NULL, NULL, NULL, NULL},

  /* ================== Server classes ================== */

{"network",        0, MXN_NETWORK_SERVER, MXR_SERVER,
				NULL, NULL, NULL, NULL, NULL},

{"", 0, 0, 0, NULL, NULL, NULL, NULL, NULL}
};

  /********************** Record Types **********************/

MX_DRIVER mx_type_list[] = {

{"list_head",      MXT_LIST_HEAD,    MXL_LIST_HEAD,      MXR_LIST_HEAD,
				&mxr_list_head_record_function_list,
				NULL,
				NULL,
				&mxr_list_head_num_record_fields,
				&mxr_list_head_rfield_def_ptr},

  /* =================== Interface types =================== */

{"soft_rs232",     MXI_232_SOFTWARE, MXI_RS232,          MXR_INTERFACE,
				&mxi_soft_rs232_record_function_list,
				NULL,
				&mxi_soft_rs232_rs232_function_list,
				&mxi_soft_rs232_num_record_fields,
				&mxi_soft_rs232_rfield_def_ptr},

{"network_rs232",  MXI_232_NETWORK,  MXI_RS232,          MXR_INTERFACE,
				&mxi_network_rs232_record_function_list,
				NULL,
				&mxi_network_rs232_rs232_function_list,
				&mxi_network_rs232_num_record_fields,
				&mxi_network_rs232_rfield_def_ptr},

#if defined(OS_UNIX) || defined(OS_CYGWIN)
{"tty",            MXI_232_TTY,      MXI_RS232,          MXR_INTERFACE,
				&mxi_tty_record_function_list,
				NULL,
				&mxi_tty_rs232_function_list,
				&mxi_tty_num_record_fields,
				&mxi_tty_rfield_def_ptr},
#endif /* OS_UNIX */

#if defined(OS_WIN32)
{"win32_com",      MXI_232_WIN32COM, MXI_RS232,          MXR_INTERFACE,
				&mxi_win32com_record_function_list,
				NULL,
				&mxi_win32com_rs232_function_list,
				&mxi_win32com_num_record_fields,
				&mxi_win32com_rfield_def_ptr},
#endif /* OS_WIN32 */

#if defined(OS_MSDOS)
{"dos_com",        MXI_232_COM,      MXI_RS232,          MXR_INTERFACE,
				&mxi_com_record_function_list,
				NULL,
				&mxi_com_rs232_function_list,
				&mxi_com_num_record_fields,
				&mxi_com_rfield_def_ptr},

{"fossil",         MXI_232_FOSSIL,   MXI_RS232,          MXR_INTERFACE,
				&mxi_fossil_record_function_list,
				NULL,
				&mxi_fossil_rs232_function_list,
				&mxi_fossil_num_record_fields,
				&mxi_fossil_rfield_def_ptr},
#endif /* OS_MSDOS */

{"ks3344",         MXI_232_KS3344,   MXI_RS232,          MXR_INTERFACE,
				&mxi_ks3344_record_function_list,
				NULL,
				&mxi_ks3344_rs232_function_list,
				&mxi_ks3344_num_record_fields,
				&mxi_ks3344_rfield_def_ptr},

#if USE_TCPIP

{"tcp232",         MXI_232_TCP232,   MXI_RS232,          MXR_INTERFACE,
				&mxi_tcp232_record_function_list,
				NULL,
				&mxi_tcp232_rs232_function_list,
				&mxi_tcp232_num_record_fields,
				&mxi_tcp232_rfield_def_ptr},
#endif /* USE_TCPIP */

#if HAVE_NI488
{"ni488",          MXI_GPIB_NI,      MXI_GPIB,           MXR_INTERFACE,
				&mxi_ni488_record_function_list,
				NULL,
				&mxi_ni488_gpib_function_list,
				&mxi_ni488_num_record_fields,
				&mxi_ni488_rfield_def_ptr},
#endif /* HAVE_NI488 */

{"k500serial",     MXI_GPIB_K500SERIAL, MXI_GPIB,         MXR_INTERFACE,
				&mxi_k500serial_record_function_list,
				NULL,
				&mxi_k500serial_gpib_function_list,
				&mxi_k500serial_num_record_fields,
				&mxi_k500serial_rfield_def_ptr},

{"soft_camac",     MXI_CAM_SOFTWARE,  MXI_CAMAC,          MXR_INTERFACE,
				&mxi_scamac_record_function_list,
				NULL,
				&mxi_scamac_camac_function_list,
				&mxi_scamac_num_record_fields,
				&mxi_scamac_record_field_def_ptr},

{"dsp6001",        MXI_CAM_DSP6001,   MXI_CAMAC,          MXR_INTERFACE,
				&mxi_dsp6001_record_function_list,
				NULL,
				&mxi_dsp6001_camac_function_list,
				&mxi_dsp6001_num_record_fields,
				&mxi_dsp6001_rfield_def_ptr},

  /* === */

{"mm3000",         MXI_GEN_MM3000,    MXI_GENERIC,        MXR_INTERFACE,
				&mxi_newport_record_function_list,
				NULL,
				NULL,
				&mxi_newport_num_record_fields,
				&mxi_newport_rfield_def_ptr},

{"mm4000",         MXI_GEN_MM4000,    MXI_GENERIC,        MXR_INTERFACE,
				&mxi_newport_record_function_list,
				NULL,
				NULL,
				&mxi_newport_num_record_fields,
				&mxi_newport_rfield_def_ptr},

{"esp",            MXI_GEN_ESP,       MXI_GENERIC,        MXR_INTERFACE,
				&mxi_newport_record_function_list,
				NULL,
				NULL,
				&mxi_newport_num_record_fields,
				&mxi_newport_rfield_def_ptr},

{"pmac",           MXI_GEN_PMAC,      MXI_GENERIC,        MXR_INTERFACE,
                                &mxi_pmac_record_function_list,
                                NULL,
                                &mxi_pmac_generic_function_list,
				&mxi_pmac_num_record_fields,
				&mxi_pmac_rfield_def_ptr},

{"compumotor_int", MXI_GEN_COMPUMOTOR, MXI_GENERIC,       MXR_INTERFACE,
				&mxi_compumotor_record_function_list,
				NULL,
				&mxi_compumotor_generic_function_list,
				&mxi_compumotor_num_record_fields,
				&mxi_compumotor_rfield_def_ptr},

{"ggcs",           MXI_GEN_GGCS,       MXI_GENERIC,       MXR_INTERFACE,
				&mxi_ggcs_record_function_list,
				NULL,
				&mxi_ggcs_generic_function_list,
				&mxi_ggcs_num_record_fields,
				&mxi_ggcs_rfield_def_ptr},

{"d8",             MXI_GEN_D8,         MXI_GENERIC,       MXR_INTERFACE,
				&mxi_d8_record_function_list,
				NULL,
				&mxi_d8_generic_function_list,
				&mxi_d8_num_record_fields,
				&mxi_d8_rfield_def_ptr},

{"vp9000",         MXI_GEN_VP9000,     MXI_GENERIC,       MXR_INTERFACE,
				&mxi_vp9000_record_function_list,
				NULL,
				&mxi_vp9000_generic_function_list,
				&mxi_vp9000_num_record_fields,
				&mxi_vp9000_rfield_def_ptr},

{"pdi40",          MXI_GEN_PDI40,      MXI_GENERIC,       MXR_INTERFACE,
				&mxi_pdi40_record_function_list,
				NULL,
				&mxi_pdi40_generic_function_list,
				&mxi_pdi40_num_record_fields,
				&mxi_pdi40_rfield_def_ptr},

{"pdi45",          MXI_GEN_PDI45,      MXI_GENERIC,       MXR_INTERFACE,
				&mxi_pdi45_record_function_list,
				NULL,
				&mxi_pdi45_generic_function_list,
				&mxi_pdi45_num_record_fields,
				&mxi_pdi45_rfield_def_ptr},
{"i8255",          MXI_GEN_8255,       MXI_GENERIC,       MXR_INTERFACE,
				&mxi_8255_record_function_list,
				NULL,
				&mxi_8255_generic_function_list,
				&mxi_8255_num_record_fields,
				&mxi_8255_rfield_def_ptr},

{"mc6821",         MXI_GEN_6821,       MXI_GENERIC,       MXR_INTERFACE,
				&mxi_6821_record_function_list,
				NULL,
				&mxi_6821_generic_function_list,
				&mxi_6821_num_record_fields,
				&mxi_6821_rfield_def_ptr},

{"am9513",          MXI_GEN_AM9513,       MXI_GENERIC,       MXR_INTERFACE,
				&mxi_am9513_record_function_list,
				NULL,
				&mxi_am9513_generic_function_list,
				&mxi_am9513_num_record_fields,
				&mxi_am9513_rfield_def_ptr},

{"hsc1",            MXI_GEN_HSC1,         MXI_GENERIC,       MXR_INTERFACE,
				&mxi_hsc1_record_function_list,
				NULL,
				&mxi_hsc1_generic_function_list,
				&mxi_hsc1_num_record_fields,
				&mxi_hsc1_rfield_def_ptr},

{"pcstep",          MXI_GEN_PCSTEP,       MXI_GENERIC,       MXR_INTERFACE,
				&mxi_pcstep_record_function_list,
				NULL,
				&mxi_pcstep_generic_function_list,
				&mxi_pcstep_num_record_fields,
				&mxi_pcstep_rfield_def_ptr},

{"ortec974",       MXI_GEN_ORTEC974,   MXI_GENERIC,       MXR_INTERFACE,
				&mxi_ortec974_record_function_list,
				NULL,
				&mxi_ortec974_generic_function_list,
				&mxi_ortec974_num_record_fields,
				&mxi_ortec974_rfield_def_ptr},

{"scipe_server",   MXI_GEN_SCIPE,      MXI_GENERIC,       MXR_INTERFACE,
				&mxi_scipe_record_function_list,
				NULL,
				&mxi_scipe_generic_function_list,
				&mxi_scipe_num_record_fields,
				&mxi_scipe_rfield_def_ptr},

{"databox",        MXI_GEN_DATABOX,    MXI_GENERIC,       MXR_INTERFACE,
				&mxi_databox_record_function_list,
				NULL,
				NULL,
				&mxi_databox_num_record_fields,
				&mxi_databox_rfield_def_ptr},

#if USE_TCPIP
{"xia_mds",        MXI_GEN_XIA_MDS,    MXI_GENERIC,       MXR_INTERFACE,
				&mxi_xia_mds_record_function_list,
				NULL,
				NULL,
				&mxi_xia_mds_num_record_fields,
				&mxi_xia_mds_rfield_def_ptr},

{"xia_network",    MXI_GEN_XIA_NETWORK, MXI_GENERIC,      MXR_INTERFACE,
				&mxi_xia_network_record_function_list,
				NULL,
				NULL,
				&mxi_xia_network_num_record_fields,
				&mxi_xia_network_rfield_def_ptr},
#endif /* USE_TCPIP */

#if HAVE_XIA_XERXES
{"xia_xerxes",     MXI_GEN_XIA_XERXES, MXI_GENERIC,       MXR_INTERFACE,
				&mxi_xia_xerxes_record_function_list,
				NULL,
				NULL,
				&mxi_xia_xerxes_num_record_fields,
				&mxi_xia_xerxes_rfield_def_ptr},
#endif /* HAVE_XIA_XERXES */

#if HAVE_ORTEC_UMCBI
{"umcbi",          MXI_GEN_UMCBI,     MXI_GENERIC,        MXR_INTERFACE,
				&mxi_umcbi_record_function_list,
				NULL,
				&mxi_umcbi_generic_function_list,
				&mxi_umcbi_num_record_fields,
				&mxi_umcbi_rfield_def_ptr},
#endif /* HAVE_ORTEC_UMCBI */

#ifdef OS_LINUX
#if HAVE_LINUX_PORTIO

{"linux_portio",   MXI_PIO_PORTIO,    MXI_PORTIO,         MXR_INTERFACE,
				&mxi_linux_portio_record_function_list,
				NULL,
				&mxi_linux_portio_portio_function_list,
				&mxi_linux_portio_num_record_fields,
				&mxi_linux_portio_rfield_def_ptr},
#endif /* HAVE_LINUX_PORTIO */
#endif /* OS_LINUX */

#if defined(OS_MSDOS) | defined(OS_WIN32)
{"dos_portio",     MXI_PIO_DOS,       MXI_PORTIO,         MXR_INTERFACE,
				&mxi_dos_portio_record_function_list,
				NULL,
				&mxi_dos_portio_portio_function_list,
				&mxi_dos_portio_num_record_fields,
				&mxi_dos_portio_rfield_def_ptr},
#endif /* OS_MSDOS | OS_WIN32 */

#ifdef OS_WIN32
#if HAVE_DRIVERLINX_PORTIO

{"dl_portio",      MXI_PIO_DRIVERLINX, MXI_PORTIO,     MXR_INTERFACE,
				&mxi_driverlinx_portio_record_function_list,
				NULL,
				&mxi_driverlinx_portio_portio_function_list,
				&mxi_driverlinx_portio_num_record_fields,
				&mxi_driverlinx_portio_rfield_def_ptr},
#endif /* HAVE_DRIVERLINX_PORTIO */
#endif /* OS_WIN32 */

#if HAVE_NI_VISA

{"vxi_memacc", MXI_VME_VXI_MEMACC,     MXI_VME,        MXR_INTERFACE,
				&mxi_vxi_memacc_record_function_list,
				NULL,
				&mxi_vxi_memacc_vme_function_list,
				&mxi_vxi_memacc_num_record_fields,
				&mxi_vxi_memacc_rfield_def_ptr},

#endif /* HAVE_NI_VISA */

  /* ==================== Device types ==================== */

{"network_ainput", MXT_AIN_NETWORK,    MXC_ANALOG_INPUT,   MXR_DEVICE,
				&mxd_network_ainput_record_function_list,
				NULL,
				&mxd_network_ainput_analog_input_function_list,
				&mxd_network_ainput_num_record_fields,
				&mxd_network_ainput_rfield_def_ptr},

{"network_aoutput",MXT_AOU_NETWORK,    MXC_ANALOG_OUTPUT,  MXR_DEVICE,
				&mxd_network_aoutput_record_function_list,
				NULL,
			&mxd_network_aoutput_analog_output_function_list,
				&mxd_network_aoutput_num_record_fields,
				&mxd_network_aoutput_rfield_def_ptr},

{"ks3512",         MXT_AIN_KS3512,    MXC_ANALOG_INPUT,   MXR_DEVICE,
				&mxd_ks3512_record_function_list,
				NULL,
				&mxd_ks3512_analog_input_function_list,
				&mxd_ks3512_num_record_fields,
				&mxd_ks3512_rfield_def_ptr},

{"ks3112",         MXT_AOU_KS3112,    MXC_ANALOG_OUTPUT,  MXR_DEVICE,
				&mxd_ks3112_record_function_list,
				NULL,
				&mxd_ks3112_analog_output_function_list,
				&mxd_ks3112_num_record_fields,
				&mxd_ks3112_rfield_def_ptr},

{"pmac_ainput",    MXT_AIN_PMAC,      MXC_ANALOG_INPUT,   MXR_DEVICE,
				&mxd_pmac_ain_record_function_list,
				NULL,
				&mxd_pmac_ain_analog_input_function_list,
				&mxd_pmac_ain_num_record_fields,
				&mxd_pmac_ain_rfield_def_ptr},

{"pmac_aoutput",   MXT_AOU_PMAC,      MXC_ANALOG_OUTPUT,  MXR_DEVICE,
				&mxd_pmac_aout_record_function_list,
				NULL,
				&mxd_pmac_aout_analog_output_function_list,
				&mxd_pmac_aout_num_record_fields,
				&mxd_pmac_aout_rfield_def_ptr},

{"mca_value",      MXT_AIN_MCA_VALUE, MXC_ANALOG_INPUT,   MXR_DEVICE,
				&mxd_mca_value_record_function_list,
				NULL,
				&mxd_mca_value_analog_input_function_list,
				&mxd_mca_value_num_record_fields,
				&mxd_mca_value_rfield_def_ptr},

{"soft_dinput",    MXT_DIN_SOFTWARE,   MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_soft_dinput_record_function_list,
				NULL,
				&mxd_soft_dinput_digital_input_function_list,
				&mxd_soft_dinput_num_record_fields,
				&mxd_soft_dinput_rfield_def_ptr},

{"soft_doutput",   MXT_DOU_SOFTWARE,   MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_soft_doutput_record_function_list,
				NULL,
				&mxd_soft_doutput_digital_output_function_list,
				&mxd_soft_doutput_num_record_fields,
				&mxd_soft_doutput_rfield_def_ptr},

{"network_dinput", MXT_DIN_NETWORK,    MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_network_dinput_record_function_list,
				NULL,
				&mxd_network_dinput_digital_input_function_list,
				&mxd_network_dinput_num_record_fields,
				&mxd_network_dinput_rfield_def_ptr},

{"network_doutput",MXT_DOU_NETWORK,    MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_network_doutput_record_function_list,
				NULL,
			&mxd_network_doutput_digital_output_function_list,
				&mxd_network_doutput_num_record_fields,
				&mxd_network_doutput_rfield_def_ptr},

{"ks3063_in",      MXT_DIN_KS3063,    MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_ks3063_in_record_function_list,
				NULL,
				&mxd_ks3063_in_digital_input_function_list,
				&mxd_ks3063_in_num_record_fields,
				&mxd_ks3063_in_rfield_def_ptr},

{"ks3063_out",     MXT_DOU_KS3063,    MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_ks3063_out_record_function_list,
				NULL,
				&mxd_ks3063_out_digital_output_function_list,
				&mxd_ks3063_out_num_record_fields,
				&mxd_ks3063_out_rfield_def_ptr},

{"i8255_in",       MXT_DIN_8255,      MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_8255_in_record_function_list,
				NULL,
				&mxd_8255_in_generic_function_list,
				&mxd_8255_in_num_record_fields,
				&mxd_8255_in_rfield_def_ptr},

{"i8255_out",      MXT_DOU_8255,      MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_8255_out_record_function_list,
				NULL,
				&mxd_8255_out_generic_function_list,
				&mxd_8255_out_num_record_fields,
				&mxd_8255_out_rfield_def_ptr},

{"mc6821_in",      MXT_DIN_6821,      MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_6821_in_record_function_list,
				NULL,
				&mxd_6821_in_digital_input_function_list,
				&mxd_6821_in_num_record_fields,
				&mxd_6821_in_rfield_def_ptr},

{"mc6821_out",     MXT_DOU_6821,      MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_6821_out_record_function_list,
				NULL,
				&mxd_6821_out_digital_output_function_list,
				&mxd_6821_out_num_record_fields,
				&mxd_6821_out_rfield_def_ptr},

{"bit_in",         MXT_DIN_BIT,       MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_bit_in_record_function_list,
				NULL,
				&mxd_bit_in_digital_input_function_list,
				&mxd_bit_in_num_record_fields,
				&mxd_bit_in_rfield_def_ptr},

{"bit_out",        MXT_DOU_BIT,       MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_bit_out_record_function_list,
				NULL,
				&mxd_bit_out_digital_output_function_list,
				&mxd_bit_out_num_record_fields,
				&mxd_bit_out_rfield_def_ptr},

{"vme_dinput",     MXT_DIN_VME,       MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_vme_din_record_function_list,
				NULL,
				&mxd_vme_din_digital_input_function_list,
				&mxd_vme_din_num_record_fields,
				&mxd_vme_din_rfield_def_ptr},

{"vme_doutput",    MXT_DOU_VME,       MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_vme_dout_record_function_list,
				NULL,
				&mxd_vme_dout_digital_output_function_list,
				&mxd_vme_dout_num_record_fields,
				&mxd_vme_dout_rfield_def_ptr},

{"pmac_dinput",    MXT_DIN_PMAC,      MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_pmac_din_record_function_list,
				NULL,
				&mxd_pmac_din_digital_input_function_list,
				&mxd_pmac_din_num_record_fields,
				&mxd_pmac_din_rfield_def_ptr},

{"pmac_doutput",   MXT_DOU_PMAC,      MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_pmac_dout_record_function_list,
				NULL,
				&mxd_pmac_dout_digital_output_function_list,
				&mxd_pmac_dout_num_record_fields,
				&mxd_pmac_dout_rfield_def_ptr},

{"compumotor_din", MXT_DIN_COMPUMOTOR,MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_compumotor_din_record_function_list,
				NULL,
				&mxd_compumotor_din_digital_input_function_list,
				&mxd_compumotor_din_num_record_fields,
				&mxd_compumotor_din_rfield_def_ptr},

{"compumotor_dout",MXT_DOU_COMPUMOTOR,MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_compumotor_dout_record_function_list,
				NULL,
				&mxd_compumotor_dout_digital_output_function_list,
				&mxd_compumotor_dout_num_record_fields,
				&mxd_compumotor_dout_rfield_def_ptr},

{"soft_motor",     MXT_MTR_SOFTWARE,  MXC_MOTOR,          MXR_DEVICE,
				&mxd_soft_motor_record_function_list,
				NULL,
				&mxd_soft_motor_motor_function_list,
				&mxd_soft_motor_num_record_fields,
				&mxd_soft_motor_rfield_def_ptr},
{"e500",           MXT_MTR_E500,      MXC_MOTOR,          MXR_DEVICE,
				&mxd_e500_record_function_list,
				NULL,
				&mxd_e500_motor_function_list,
				&mxd_e500_num_record_fields,
				&mxd_e500_rfield_def_ptr},
{"smc24",          MXT_MTR_SMC24,     MXC_MOTOR,          MXR_DEVICE,
				&mxd_smc24_record_function_list,
				NULL,
				&mxd_smc24_motor_function_list,
				&mxd_smc24_num_record_fields,
				&mxd_smc24_rfield_def_ptr},
{"panther_hi",     MXT_MTR_PANTHER_HI, MXC_MOTOR,          MXR_DEVICE,
				&mxd_panther_record_function_list,
				NULL,
				&mxd_panther_motor_function_list,
				&mxd_panther_hi_num_record_fields,
				&mxd_panther_hi_rfield_def_ptr},
{"panther_he",     MXT_MTR_PANTHER_HE, MXC_MOTOR,          MXR_DEVICE,
				&mxd_panther_record_function_list,
				NULL,
				&mxd_panther_motor_function_list,
				&mxd_panther_he_num_record_fields,
				&mxd_panther_he_rfield_def_ptr},
{"pdi40_motor",    MXT_MTR_PDI40,     MXC_MOTOR,          MXR_DEVICE,
				&mxd_pdi40motor_record_function_list,
				NULL,
				&mxd_pdi40motor_motor_function_list,
				&mxd_pdi40motor_num_record_fields,
				&mxd_pdi40motor_rfield_def_ptr},
{"mmc32",          MXT_MTR_MMC32,     MXC_MOTOR,          MXR_DEVICE,
				&mxd_mmc32_record_function_list,
				NULL,
				&mxd_mmc32_motor_function_list,
				&mxd_mmc32_num_record_fields,
				&mxd_mmc32_rfield_def_ptr},
{"pm304",          MXT_MTR_PM304,     MXC_MOTOR,          MXR_DEVICE,
				&mxd_pm304_record_function_list,
				NULL,
				&mxd_pm304_motor_function_list,
				&mxd_pm304_num_record_fields,
				&mxd_pm304_rfield_def_ptr},
{"pmac_motor",     MXT_MTR_PMAC,      MXC_MOTOR,          MXR_DEVICE,
                                &mxd_pmac_record_function_list,
                                NULL,
                                &mxd_pmac_motor_function_list,
				&mxd_pmac_num_record_fields,
				&mxd_pmac_rfield_def_ptr},
{"compumotor",     MXT_MTR_COMPUMOTOR, MXC_MOTOR,         MXR_DEVICE,
				&mxd_compumotor_record_function_list,
				NULL,
				&mxd_compumotor_motor_function_list,
				&mxd_compumotor_num_record_fields,
				&mxd_compumotor_rfield_def_ptr},
{"network_motor",  MXT_MTR_NETWORK,   MXC_MOTOR,          MXR_DEVICE,
				&mxd_network_motor_record_function_list,
				NULL,
				&mxd_network_motor_motor_function_list,
				&mxd_network_motor_num_record_fields,
				&mxd_network_motor_rfield_def_ptr},
{"ggcs_motor",     MXT_MTR_GGCS,      MXC_MOTOR,          MXR_DEVICE,
				&mxd_ggcs_motor_record_function_list,
				NULL,
				&mxd_ggcs_motor_motor_function_list,
				&mxd_ggcs_motor_num_record_fields,
				&mxd_ggcs_motor_rfield_def_ptr},

{"d8_motor",       MXT_MTR_D8,        MXC_MOTOR,          MXR_DEVICE,
				&mxd_d8_motor_record_function_list,
				NULL,
				&mxd_d8_motor_motor_function_list,
				&mxd_d8_motor_num_record_fields,
				&mxd_d8_motor_rfield_def_ptr},

{"vp9000_motor",   MXT_MTR_VP9000,    MXC_MOTOR,          MXR_DEVICE,
				&mxd_vp9000_record_function_list,
				NULL,
				&mxd_vp9000_motor_function_list,
				&mxd_vp9000_num_record_fields,
				&mxd_vp9000_rfield_def_ptr},

{"e662",           MXT_MTR_E662,      MXC_MOTOR,          MXR_DEVICE,
				&mxd_e662_record_function_list,
				NULL,
				&mxd_e662_motor_function_list,
				&mxd_e662_num_record_fields,
				&mxd_e662_rfield_def_ptr},

{"hsc1_motor",     MXT_MTR_HSC1,      MXC_MOTOR,          MXR_DEVICE,
				&mxd_hsc1_record_function_list,
				NULL,
				&mxd_hsc1_motor_function_list,
				&mxd_hsc1_num_record_fields,
				&mxd_hsc1_rfield_def_ptr},

{"stp100_motor",   MXT_MTR_STP100,    MXC_MOTOR,          MXR_DEVICE,
				&mxd_stp100_motor_record_function_list,
				NULL,
				&mxd_stp100_motor_motor_function_list,
				&mxd_stp100_motor_num_record_fields,
				&mxd_stp100_motor_rfield_def_ptr},

{"pcstep_motor",   MXT_MTR_PCSTEP,    MXC_MOTOR,          MXR_DEVICE,
				&mxd_pcstep_record_function_list,
				NULL,
				&mxd_pcstep_motor_function_list,
				&mxd_pcstep_num_record_fields,
				&mxd_pcstep_rfield_def_ptr},

{"scipe_motor",    MXT_MTR_SCIPE,     MXC_MOTOR,          MXR_DEVICE,
				&mxd_scipe_motor_record_function_list,
				NULL,
				&mxd_scipe_motor_motor_function_list,
				&mxd_scipe_motor_num_record_fields,
				&mxd_scipe_motor_rfield_def_ptr},

/* The Newport MM3000 and MM4000 share a driver. */

{"mm3000_motor",   MXT_MTR_MM3000,    MXC_MOTOR,          MXR_DEVICE,
				&mxd_newport_record_function_list,
				NULL,
				&mxd_newport_motor_function_list,
				&mxd_mm3000_num_record_fields,
				&mxd_mm3000_rfield_def_ptr},
{"mm4000_motor",   MXT_MTR_MM4000,    MXC_MOTOR,          MXR_DEVICE,
				&mxd_newport_record_function_list,
				NULL,
				&mxd_newport_motor_function_list,
				&mxd_esp_mm4000_num_record_fields,
				&mxd_esp_mm4000_rfield_def_ptr},
{"esp_motor",      MXT_MTR_ESP,       MXC_MOTOR,          MXR_DEVICE,
				&mxd_newport_record_function_list,
				NULL,
				&mxd_newport_motor_function_list,
				&mxd_esp_mm4000_num_record_fields,
				&mxd_esp_mm4000_rfield_def_ptr},


{"dac_motor",      MXT_MTR_DAC_MOTOR, MXC_MOTOR,          MXR_DEVICE,
				&mxd_dac_motor_record_function_list,
				NULL,
				&mxd_dac_motor_motor_function_list,
				&mxd_dac_motor_num_record_fields,
				&mxd_dac_motor_rfield_def_ptr},
{"ls330_motor",    MXT_MTR_LAKESHORE330, MXC_MOTOR,       MXR_DEVICE,
				&mxd_ls330_motor_record_function_list,
				NULL,
				&mxd_ls330_motor_motor_function_list,
				&mxd_ls330_motor_num_record_fields,
				&mxd_ls330_motor_rfield_def_ptr},

{"databox_motor",  MXT_MTR_DATABOX,   MXC_MOTOR,       MXR_DEVICE,
				&mxd_databox_motor_record_function_list,
				NULL,
				&mxd_databox_motor_motor_function_list,
				&mxd_databox_motor_num_record_fields,
				&mxd_databox_motor_rfield_def_ptr},

{"disabled_motor", MXT_MTR_DISABLED,  MXC_MOTOR,          MXR_DEVICE,
				&mxd_disabled_motor_record_function_list,
				NULL,
				&mxd_disabled_motor_motor_function_list,
				&mxd_disabled_motor_num_record_fields,
				&mxd_disabled_motor_rfield_def_ptr},

/* Pseudo motors. */

{"energy_motor",   MXT_MTR_ENERGY,    MXC_MOTOR,          MXR_DEVICE,
				&mxd_energy_motor_record_function_list,
				NULL,
				&mxd_energy_motor_motor_function_list,
				&mxd_energy_motor_num_record_fields,
				&mxd_energy_motor_rfield_def_ptr},
{"wavelength_motor", MXT_MTR_WAVELENGTH, MXC_MOTOR,       MXR_DEVICE,
				&mxd_wavelen_motor_record_function_list,
				NULL,
				&mxd_wavelen_motor_motor_function_list,
				&mxd_wavelen_motor_num_record_fields,
				&mxd_wavelen_motor_rfield_def_ptr},
{"wavenumber_motor", MXT_MTR_WAVENUMBER, MXC_MOTOR,       MXR_DEVICE,
				&mxd_wavenum_motor_record_function_list,
				NULL,
				&mxd_wavenum_motor_motor_function_list,
				&mxd_wavenum_motor_num_record_fields,
				&mxd_wavenum_motor_rfield_def_ptr},
{"slit_motor",     MXT_MTR_SLIT_MOTOR, MXC_MOTOR,         MXR_DEVICE,
				&mxd_slit_motor_record_function_list,
				NULL,
				&mxd_slit_motor_motor_function_list,
				&mxd_slit_motor_num_record_fields,
				&mxd_slit_motor_rfield_def_ptr},
{"translation_mtr", MXT_MTR_TRANSLATION, MXC_MOTOR,       MXR_DEVICE,
				&mxd_trans_motor_record_function_list,
				NULL,
				&mxd_trans_motor_motor_function_list,
				&mxd_trans_motor_num_record_fields,
				&mxd_trans_motor_rfield_def_ptr},
{"xafs_wavenumber", MXT_MTR_XAFS_WAVENUMBER,
                                      MXC_MOTOR,          MXR_DEVICE,
				&mxd_xafswn_motor_record_function_list,
				NULL,
				&mxd_xafswn_motor_motor_function_list,
				&mxd_xafswn_motor_num_record_fields,
				&mxd_xafswn_motor_rfield_def_ptr},
{"delta_motor",    MXT_MTR_DELTA,     MXC_MOTOR,          MXR_DEVICE,
				&mxd_delta_motor_record_function_list,
				NULL,
				&mxd_delta_motor_motor_function_list,
				&mxd_delta_motor_num_record_fields,
				&mxd_delta_motor_rfield_def_ptr},
{"elapsed_time",   MXT_MTR_ELAPSED_TIME, MXC_MOTOR,       MXR_DEVICE,
				&mxd_elapsed_time_record_function_list,
				NULL,
				&mxd_elapsed_time_motor_function_list,
				&mxd_elapsed_time_num_record_fields,
				&mxd_elapsed_time_rfield_def_ptr},
{"monochromator",  MXT_MTR_MONOCHROMATOR, MXC_MOTOR,      MXR_DEVICE,
				&mxd_monochromator_record_function_list,
				NULL,
				&mxd_monochromator_motor_function_list,
				&mxd_monochromator_num_record_fields,
				&mxd_monochromator_rfield_def_ptr},
{"linear_function", MXT_MTR_LINEAR_FUNCTION, MXC_MOTOR,   MXR_DEVICE,
				&mxd_linear_function_record_function_list,
				NULL,
				&mxd_linear_function_motor_function_list,
				&mxd_linear_function_num_record_fields,
				&mxd_linear_function_rfield_def_ptr},
{"table_motor",    MXT_MTR_TABLE,     MXC_MOTOR,   MXR_DEVICE,
				&mxd_table_motor_record_function_list,
				NULL,
				&mxd_table_motor_motor_function_list,
				&mxd_table_motor_num_record_fields,
				&mxd_table_motor_rfield_def_ptr},
{"theta_2theta",   MXT_MTR_THETA_2THETA,     MXC_MOTOR,   MXR_DEVICE,
				&mxd_theta_2theta_motor_record_function_list,
				NULL,
				&mxd_theta_2theta_motor_motor_function_list,
				&mxd_theta_2theta_motor_num_record_fields,
				&mxd_theta_2theta_motor_rfield_def_ptr},
{"q_motor",        MXT_MTR_Q,                MXC_MOTOR,   MXR_DEVICE,
				&mxd_q_motor_record_function_list,
				NULL,
				&mxd_q_motor_motor_function_list,
				&mxd_q_motor_num_record_fields,
				&mxd_q_motor_rfield_def_ptr},
{"segmented_move", MXT_MTR_SEGMENTED_MOVE,   MXC_MOTOR,   MXR_DEVICE,
				&mxd_segmented_move_record_function_list,
				NULL,
				&mxd_segmented_move_motor_function_list,
				&mxd_segmented_move_num_record_fields,
				&mxd_segmented_move_rfield_def_ptr},
{"tangent_arm",    MXT_MTR_TANGENT_ARM,      MXC_MOTOR,   MXR_DEVICE,
				&mxd_tangent_arm_record_function_list,
				NULL,
				&mxd_tangent_arm_motor_function_list,
				&mxd_tangent_arm_num_record_fields,
				&mxd_tangent_arm_rfield_def_ptr},
{"sine_arm",       MXT_MTR_SINE_ARM,         MXC_MOTOR,   MXR_DEVICE,
				&mxd_tangent_arm_record_function_list,
				NULL,
				&mxd_tangent_arm_motor_function_list,
				&mxd_tangent_arm_num_record_fields,
				&mxd_tangent_arm_rfield_def_ptr},
{"aframe_det_motor", MXT_MTR_AFRAME_DETECTOR_MOTOR, MXC_MOTOR, MXR_DEVICE,
				&mxd_aframe_det_motor_record_function_list,
				NULL,
				&mxd_aframe_det_motor_motor_function_list,
				&mxd_aframe_det_motor_num_record_fields,
				&mxd_aframe_det_motor_rfield_def_ptr},

{"aps_10id_motor", MXT_MTR_APS_10ID,     MXC_MOTOR,       MXR_DEVICE,
				&mxd_aps_10id_motor_record_function_list,
				NULL,
				&mxd_aps_10id_motor_motor_function_list,
				&mxd_aps_10id_motor_num_record_fields,
				&mxd_aps_10id_motor_rfield_def_ptr},

#if HAVE_EPICS
{"aps_18id_motor", MXT_MTR_APS_18ID,     MXC_MOTOR,       MXR_DEVICE,
				&mxd_aps_18id_motor_record_function_list,
				NULL,
				&mxd_aps_18id_motor_motor_function_list,
				&mxd_aps_18id_motor_num_record_fields,
				&mxd_aps_18id_motor_rfield_def_ptr},
#endif

{"compumotor_lin", MXT_MTR_COMPUMOTOR_LINEAR, MXC_MOTOR, MXR_DEVICE,
				&mxd_compumotor_linear_record_function_list,
				NULL,
				&mxd_compumotor_linear_motor_function_list,
				&mxd_compumotor_linear_num_record_fields,
				&mxd_compumotor_linear_rfield_def_ptr},

{"compumotor_trans", MXT_MTR_COMPUMOTOR_TRANSLATION, MXC_MOTOR, MXR_DEVICE,
				&mxd_compumotor_trans_record_function_list,
				NULL,
				&mxd_compumotor_trans_motor_function_list,
				&mxd_compumotor_trans_num_record_fields,
				&mxd_compumotor_trans_rfield_def_ptr},

#if HAVE_EPICS

{"epics_rs232",    MXI_232_EPICS,    MXI_RS232,          MXR_INTERFACE,
				&mxi_epics_rs232_record_function_list,
				NULL,
				&mxi_epics_rs232_rs232_function_list,
				&mxi_epics_rs232_num_record_fields,
				&mxi_epics_rs232_rfield_def_ptr},
{"epics_gpib",     MXI_GPIB_EPICS,   MXI_GPIB,           MXR_INTERFACE,
				&mxi_epics_gpib_record_function_list,
				NULL,
				&mxi_epics_gpib_gpib_function_list,
				&mxi_epics_gpib_num_record_fields,
				&mxi_epics_gpib_rfield_def_ptr},
{"epics_motor",    MXT_MTR_EPICS,     MXC_MOTOR,          MXR_DEVICE,
				&mxd_epics_motor_record_function_list,
				NULL,
				&mxd_epics_motor_motor_function_list,
				&mxd_epics_motor_num_record_fields,
				&mxd_epics_motor_rfield_def_ptr},
{"epics_scaler",   MXT_SCL_EPICS,     MXC_SCALER,         MXR_DEVICE,
				&mxd_epics_scaler_record_function_list,
				NULL,
				&mxd_epics_scaler_scaler_function_list,
				&mxd_epics_scaler_num_record_fields,
				&mxd_epics_scaler_rfield_def_ptr},
{"epics_timer",    MXT_TIM_EPICS,     MXC_TIMER,          MXR_DEVICE,
				&mxd_epics_timer_record_function_list,
				NULL,
				&mxd_epics_timer_timer_function_list,
				&mxd_epics_timer_num_record_fields,
				&mxd_epics_timer_rfield_def_ptr},

/* pmac_tc_motor and pmac_bio_motor share the same driver. */

{"pmac_tc_motor",  MXT_MTR_PMAC_EPICS_TC, MXC_MOTOR,      MXR_DEVICE,
				&mxd_pmac_tc_motor_record_function_list,
				NULL,
				&mxd_pmac_tc_motor_motor_function_list,
				&mxd_pmac_tc_motor_num_record_fields,
				&mxd_pmac_tc_motor_rfield_def_ptr},
{"pmac_bio_motor", MXT_MTR_PMAC_EPICS_BIO, MXC_MOTOR,      MXR_DEVICE,
				&mxd_pmac_bio_motor_record_function_list,
				NULL,
				&mxd_pmac_bio_motor_motor_function_list,
				&mxd_pmac_bio_motor_num_record_fields,
				&mxd_pmac_bio_motor_rfield_def_ptr},

{"aps_gap",        MXT_MTR_APS_GAP,   MXC_MOTOR,          MXR_DEVICE,
				&mxd_aps_gap_record_function_list,
				NULL,
				&mxd_aps_gap_motor_function_list,
				&mxd_aps_gap_num_record_fields,
				&mxd_aps_gap_record_field_def_ptr},

#endif /* HAVE_EPICS */

#if HAVE_LINUX_VME58

{"vme58",          MXI_GEN_VME58,     MXI_GENERIC,       MXR_INTERFACE,
				&mxi_vme58_record_function_list,
				NULL,
				&mxi_vme58_generic_function_list,
				&mxi_vme58_num_record_fields,
				&mxi_vme58_rfield_def_ptr},

{"vme58_motor",    MXT_MTR_VME58,     MXC_MOTOR,          MXR_DEVICE,
				&mxd_vme58_record_function_list,
				NULL,
				&mxd_vme58_motor_function_list,
				&mxd_vme58_num_record_fields,
				&mxd_vme58_rfield_def_ptr},

#endif /* HAVE_LINUX_VME58 */

#if HAVE_PCMOTION32

{"pcmotion32",      MXI_GEN_PCMOTION32,  MXI_GENERIC,       MXR_INTERFACE,
				&mxi_pcmotion32_record_function_list,
				NULL,
				&mxi_pcmotion32_generic_function_list,
				&mxi_pcmotion32_num_record_fields,
				&mxi_pcmotion32_rfield_def_ptr},

{"pcmotion32_motor", MXT_MTR_PCMOTION32, MXC_MOTOR,       MXR_DEVICE,
				&mxd_pcmotion32_record_function_list,
				NULL,
				&mxd_pcmotion32_motor_function_list,
				&mxd_pcmotion32_num_record_fields,
				&mxd_pcmotion32_rfield_def_ptr},

#endif /* HAVE_PCMOTION32 */

{"encoder_soft",   MXT_ENC_SOFTWARE,  MXC_ENCODER,        MXR_DEVICE,
				NULL, NULL, NULL, NULL, NULL},
{"ks3640",         MXT_ENC_KS3640,    MXC_ENCODER,        MXR_DEVICE,
				&mxd_ks3640_record_function_list,
				NULL,
				&mxd_ks3640_encoder_function_list,
				&mxd_ks3640_num_record_fields,
				&mxd_ks3640_rfield_def_ptr},

{"soft_scaler",    MXT_SCL_SOFTWARE,  MXC_SCALER,         MXR_DEVICE,
				&mxd_soft_scaler_record_function_list,
				NULL,
				&mxd_soft_scaler_scaler_function_list,
				&mxd_soft_scaler_num_record_fields,
				&mxd_soft_scaler_rfield_def_ptr},

{"network_scaler", MXT_SCL_NETWORK,   MXC_SCALER,         MXR_DEVICE,
				&mxd_network_scaler_record_function_list,
				NULL,
				&mxd_network_scaler_scaler_function_list,
				&mxd_network_scaler_num_record_fields,
				&mxd_network_scaler_rfield_def_ptr},

{"autoscale_scaler", MXT_SCL_AUTOSCALE, MXC_SCALER,       MXR_DEVICE,
				&mxd_autoscale_scaler_record_function_list,
				NULL,
				&mxd_autoscale_scaler_scaler_function_list,
				&mxd_autoscale_scaler_num_record_fields,
				&mxd_autoscale_scaler_rfield_def_ptr},

{"gain_tracking",  MXT_SCL_GAIN_TRACKING, MXC_SCALER,       MXR_DEVICE,
				&mxd_gain_tracking_scaler_record_function_list,
				NULL,
				&mxd_gain_tracking_scaler_scaler_function_list,
				&mxd_gain_tracking_scaler_num_record_fields,
				&mxd_gain_tracking_scaler_rfield_def_ptr},

{"scaler_function", MXT_SCL_SCALER_FUNCTION, MXC_SCALER,       MXR_DEVICE,
				&mxd_scaler_function_record_function_list,
				NULL,
				&mxd_scaler_function_scaler_function_list,
				&mxd_scaler_function_num_record_fields,
				&mxd_scaler_function_rfield_def_ptr},

/* The DSP QS450 and the KS 3610 share a driver. */

{"qs450",          MXT_SCL_QS450,     MXC_SCALER,         MXR_DEVICE,
				&mxd_qs450_record_function_list,
				NULL,
				&mxd_qs450_scaler_function_list,
				&mxd_qs450_num_record_fields,
				&mxd_qs450_record_field_def_ptr},

{"ks3610",         MXT_SCL_KS3610,    MXC_SCALER,         MXR_DEVICE,
				&mxd_qs450_record_function_list,
				NULL,
				&mxd_qs450_scaler_function_list,
				&mxd_ks3610_num_record_fields,
				&mxd_ks3610_record_field_def_ptr},

{"ortec974_scaler", MXT_SCL_ORTEC974, MXC_SCALER,         MXR_DEVICE,
				&mxd_ortec974_scaler_record_function_list,
				NULL,
				&mxd_ortec974_scaler_scaler_function_list,
				&mxd_ortec974_scaler_num_record_fields,
				&mxd_ortec974_scaler_rfield_def_ptr},

{"ortec974_timer", MXT_TIM_ORTEC974,  MXC_TIMER,          MXR_DEVICE,
				&mxd_ortec974_timer_record_function_list,
				NULL,
				&mxd_ortec974_timer_timer_function_list,
				&mxd_ortec974_timer_num_record_fields,
				&mxd_ortec974_timer_rfield_def_ptr},

{"scipe_scaler",   MXT_SCL_SCIPE,     MXC_SCALER,         MXR_DEVICE,
				&mxd_scipe_scaler_record_function_list,
				NULL,
				&mxd_scipe_scaler_scaler_function_list,
				&mxd_scipe_scaler_num_record_fields,
				&mxd_scipe_scaler_rfield_def_ptr},

{"scipe_timer",    MXT_TIM_SCIPE,     MXC_TIMER,          MXR_DEVICE,
				&mxd_scipe_timer_record_function_list,
				NULL,
				&mxd_scipe_timer_timer_function_list,
				&mxd_scipe_timer_num_record_fields,
				&mxd_scipe_timer_rfield_def_ptr},

{"mca_channel",    MXT_SCL_MCA_CHANNEL, MXC_SCALER,       MXR_DEVICE,
				&mxd_mca_channel_record_function_list,
				NULL,
				&mxd_mca_channel_scaler_function_list,
				&mxd_mca_channel_num_record_fields,
				&mxd_mca_channel_rfield_def_ptr},

{"mca_roi_integral", MXT_SCL_MCA_ROI_INTEGRAL, MXC_SCALER, MXR_DEVICE,
				&mxd_mca_roi_integral_record_function_list,
				NULL,
				&mxd_mca_roi_integral_scaler_function_list,
				&mxd_mca_roi_integral_num_record_fields,
				&mxd_mca_roi_integral_rfield_def_ptr},

{"mca_alt_time",   MXT_SCL_MCA_ALTERNATE_TIME, MXC_SCALER, MXR_DEVICE,
				&mxd_mca_alt_time_record_function_list,
				NULL,
				&mxd_mca_alt_time_scaler_function_list,
				&mxd_mca_alt_time_num_record_fields,
				&mxd_mca_alt_time_rfield_def_ptr},

{"mcs_scaler",     MXT_SCL_MCS,       MXC_SCALER,         MXR_DEVICE,
				&mxd_mcs_scaler_record_function_list,
				NULL,
				&mxd_mcs_scaler_scaler_function_list,
				&mxd_mcs_scaler_num_record_fields,
				&mxd_mcs_scaler_rfield_def_ptr},

{"databox_scaler", MXT_SCL_DATABOX,   MXC_SCALER,         MXR_DEVICE,
				&mxd_databox_scaler_record_function_list,
				NULL,
				&mxd_databox_scaler_scaler_function_list,
				&mxd_databox_scaler_num_record_fields,
				&mxd_databox_scaler_rfield_def_ptr},

{"mcs_encoder",    MXT_MCE_MCS,       MXC_MULTICHANNEL_ENCODER, MXR_DEVICE,
				&mxd_mcs_encoder_record_function_list,
				NULL,
				&mxd_mcs_encoder_mce_function_list,
				&mxd_mcs_encoder_num_record_fields,
				&mxd_mcs_encoder_rfield_def_ptr},

{"databox_encoder",MXT_MCE_DATABOX,   MXC_MULTICHANNEL_ENCODER, MXR_DEVICE,
				&mxd_databox_encoder_record_function_list,
				NULL,
				&mxd_databox_encoder_mce_function_list,
				&mxd_databox_encoder_num_record_fields,
				&mxd_databox_encoder_rfield_def_ptr},

{"network_mce",    MXT_MCE_NETWORK,   MXC_MULTICHANNEL_ENCODER, MXR_DEVICE,
				&mxd_network_mce_record_function_list,
				NULL,
				&mxd_network_mce_mce_function_list,
				&mxd_network_mce_num_record_fields,
				&mxd_network_mce_rfield_def_ptr},

{"soft_timer",     MXT_TIM_SOFTWARE,  MXC_TIMER,          MXR_DEVICE,
				&mxd_soft_timer_record_function_list,
				NULL,
				&mxd_soft_timer_timer_function_list,
				&mxd_soft_timer_num_record_fields,
				&mxd_soft_timer_rfield_def_ptr},

{"network_timer",  MXT_TIM_NETWORK,   MXC_TIMER,          MXR_DEVICE,
				&mxd_network_timer_record_function_list,
				NULL,
				&mxd_network_timer_timer_function_list,
				&mxd_network_timer_num_record_fields,
				&mxd_network_timer_rfield_def_ptr},

{"timer_fanout",   MXT_TIM_FANOUT,    MXC_TIMER,          MXR_DEVICE,
				&mxd_timer_fanout_record_function_list,
				NULL,
				&mxd_timer_fanout_timer_function_list,
				&mxd_timer_fanout_num_record_fields,
				&mxd_timer_fanout_rfield_def_ptr},

{"rtc018",         MXT_TIM_RTC018,    MXC_TIMER,          MXR_DEVICE,
				&mxd_rtc018_record_function_list,
				NULL,
				&mxd_rtc018_timer_function_list,
				&mxd_rtc018_num_record_fields,
				&mxd_rtc018_rfield_def_ptr},

{"mca_timer",      MXT_TIM_MCA,       MXC_TIMER,          MXR_DEVICE,
				&mxd_mca_timer_record_function_list,
				NULL,
				&mxd_mca_timer_timer_function_list,
				&mxd_mca_timer_num_record_fields,
				&mxd_mca_timer_rfield_def_ptr},

{"mcs_timer",      MXT_TIM_MCS,       MXC_TIMER,          MXR_DEVICE,
				&mxd_mcs_timer_record_function_list,
				NULL,
				&mxd_mcs_timer_timer_function_list,
				&mxd_mcs_timer_num_record_fields,
				&mxd_mcs_timer_rfield_def_ptr},

{"databox_timer",  MXT_TIM_DATABOX,   MXC_TIMER,          MXR_DEVICE,
				&mxd_databox_timer_record_function_list,
				NULL,
				&mxd_databox_timer_timer_function_list,
				&mxd_databox_timer_num_record_fields,
				&mxd_databox_timer_rfield_def_ptr},

{"soft_amplifier", MXT_AMP_SOFTWARE,  MXC_AMPLIFIER,      MXR_DEVICE,
				&mxd_soft_amplifier_record_function_list,
				NULL,
				&mxd_soft_amplifier_amplifier_function_list,
				&mxd_soft_amplifier_num_record_fields,
				&mxd_soft_amplifier_rfield_def_ptr},

{"net_amplifier",  MXT_AMP_NETWORK,   MXC_AMPLIFIER,      MXR_DEVICE,
				&mxd_network_amplifier_record_function_list,
				NULL,
				&mxd_network_amplifier_amplifier_function_list,
				&mxd_network_amplifier_num_record_fields,
				&mxd_network_amplifier_rfield_def_ptr},

{"keithley428",    MXT_AMP_KEITHLEY428, MXC_AMPLIFIER,    MXR_DEVICE,
				&mxd_keithley428_record_function_list,
				NULL,
				&mxd_keithley428_amplifier_function_list,
				&mxd_keithley428_num_record_fields,
				&mxd_keithley428_rfield_def_ptr},

{"soft_mca",       MXT_MCA_SOFTWARE,  MXC_MULTICHANNEL_ANALYZER, MXR_DEVICE,
				&mxd_soft_mca_record_function_list,
				NULL,
				&mxd_soft_mca_mca_function_list,
				&mxd_soft_mca_num_record_fields,
				&mxd_soft_mca_rfield_def_ptr},

{"network_mca",    MXT_MCA_NETWORK,   MXC_MULTICHANNEL_ANALYZER, MXR_DEVICE,
				&mxd_network_mca_record_function_list,
				NULL,
				&mxd_network_mca_mca_function_list,
				&mxd_network_mca_num_record_fields,
				&mxd_network_mca_rfield_def_ptr},

#if USE_TCPIP || HAVE_XIA_XERXES

{"xia_dxp_mca",    MXT_MCA_XIA_DXP,   MXC_MULTICHANNEL_ANALYZER, MXR_DEVICE,
				&mxd_xia_dxp_record_function_list,
				NULL,
				&mxd_xia_dxp_mca_function_list,
				&mxd_xia_dxp_num_record_fields,
				&mxd_xia_dxp_rfield_def_ptr},

{"xia_dxp_input",  MXT_AIN_XIA_DXP,   MXC_ANALOG_INPUT,        MXR_DEVICE,
				&mxd_xia_dxp_input_record_function_list,
				NULL,
				&mxd_xia_dxp_input_analog_input_function_list,
				&mxd_xia_dxp_input_num_record_fields,
				&mxd_xia_dxp_input_rfield_def_ptr},

{"xia_dxp_sum",    MXT_AIN_XIA_DXP_SUM, MXC_ANALOG_INPUT,      MXR_DEVICE,
				&mxd_xia_dxp_sum_record_function_list,
				NULL,
				&mxd_xia_dxp_sum_analog_input_function_list,
				&mxd_xia_dxp_sum_num_record_fields,
				&mxd_xia_dxp_sum_rfield_def_ptr},

#endif /* USE_TCPIP || HAVE_XIA_XERXES */


{"soft_mcs",       MXT_MCS_SOFTWARE,  MXC_MULTICHANNEL_SCALER, MXR_DEVICE,
				&mxd_soft_mcs_record_function_list,
				NULL,
				&mxd_soft_mcs_mcs_function_list,
				&mxd_soft_mcs_num_record_fields,
				&mxd_soft_mcs_rfield_def_ptr},

{"network_mcs",    MXT_MCS_NETWORK,   MXC_MULTICHANNEL_SCALER, MXR_DEVICE,
				&mxd_network_mcs_record_function_list,
				NULL,
				&mxd_network_mcs_mcs_function_list,
				&mxd_network_mcs_num_record_fields,
				&mxd_network_mcs_rfield_def_ptr},

{"databox_mcs",    MXT_MCS_DATABOX,   MXC_MULTICHANNEL_SCALER, MXR_DEVICE,
				&mxd_databox_mcs_record_function_list,
				NULL,
				&mxd_databox_mcs_mcs_function_list,
				&mxd_databox_mcs_num_record_fields,
				&mxd_databox_mcs_rfield_def_ptr},

{"sis3801",        MXT_MCS_SIS3801,   MXC_MULTICHANNEL_SCALER, MXR_DEVICE,
				&mxd_sis3801_record_function_list,
				NULL,
				&mxd_sis3801_mcs_function_list,
				&mxd_sis3801_num_record_fields,
				&mxd_sis3801_rfield_def_ptr},

{"generic_relay",  MXT_RLY_GENERIC,   MXC_RELAY,          MXR_DEVICE,
				&mxd_generic_relay_record_function_list,
				NULL,
				&mxd_generic_relay_relay_function_list,
				&mxd_generic_relay_num_record_fields,
				&mxd_generic_relay_rfield_def_ptr},

{"blind_relay",    MXT_RLY_BLIND,     MXC_RELAY,          MXR_DEVICE,
				&mxd_blind_relay_record_function_list,
				NULL,
				&mxd_blind_relay_relay_function_list,
				&mxd_blind_relay_num_record_fields,
				&mxd_blind_relay_rfield_def_ptr},

{"adc_table",      MXT_TAB_ADC,       MXC_TABLE,          MXR_DEVICE,
				&mxd_adc_table_record_function_list,
				NULL,
				&mxd_adc_table_table_function_list,
				&mxd_adc_table_num_record_fields,
				&mxd_adc_table_rfield_def_ptr},

{"autoscale_filter",MXT_AUT_FILTER,   MXC_AUTOSCALE,      MXR_DEVICE,
				&mxd_auto_filter_record_function_list,
				NULL,
				&mxd_auto_filter_autoscale_function_list,
				&mxd_auto_filter_num_record_fields,
				&mxd_auto_filter_rfield_def_ptr},

{"autoscale_amp",  MXT_AUT_AMPLIFIER, MXC_AUTOSCALE,      MXR_DEVICE,
				&mxd_auto_amplifier_record_function_list,
				NULL,
				&mxd_auto_amplifier_autoscale_function_list,
				&mxd_auto_amplifier_num_record_fields,
				&mxd_auto_amplifier_rfield_def_ptr},

{"autoscale_net",  MXT_AUT_NETWORK,   MXC_AUTOSCALE,      MXR_DEVICE,
				&mxd_auto_network_record_function_list,
				NULL,
				&mxd_auto_network_autoscale_function_list,
				&mxd_auto_network_num_record_fields,
				&mxd_auto_network_rfield_def_ptr},

{"auto_filter_amp",MXT_AUT_FILTER_AMPLIFIER, MXC_AUTOSCALE, MXR_DEVICE,
				&mxd_auto_filter_amp_record_function_list,
				NULL,
				&mxd_auto_filter_amp_autoscale_function_list,
				&mxd_auto_filter_amp_num_record_fields,
				&mxd_auto_filter_amp_rfield_def_ptr},

#if 0
{"pdi45_adc",      MXT_AIN_PDI45,     MXC_ANALOG_INPUT,   MXR_DEVICE,
				&mxd_pdi45_record_function_list,
				NULL,
				&mxd_pdi45adc_analog_input_function_list,
				NULL, NULL},
{"pdi45_dac",      MXT_AOU_PDI45,     MXC_ANALOG_OUTPUT,  MXR_DEVICE,
				&mxd_pdi45_record_function_list,
				NULL,
				&mxd_pdi45dac_analog_output_function_list,
				NULL, NULL},
{"pdi45_din",      MXT_DIN_PDI45,     MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_pdi45_record_function_list,
				NULL,
				&mxd_pdi45din_digital_input_function_list,
				NULL, NULL},
{"pdi45_dout",     MXT_DOU_PDI45,     MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_pdi45_record_function_list,
				NULL,
				&mxd_pdi45dout_digital_output_function_list,
				NULL, NULL},
{"pdi45_scaler",   MXT_SCL_PDI45,     MXC_SCALER,         MXR_DEVICE,
				&mxd_pdi45_record_function_list,
				NULL,
				&mxd_pdi45scaler_scaler_function_list,
				NULL, NULL},
{"pdi45_timer",    MXT_TIM_PDI45,     MXC_TIMER,          MXR_DEVICE,
				&mxd_pdi45_record_function_list,
				NULL,
				&mxd_pdi45timer_timer_function_list,
				NULL, NULL},
#endif

#if HAVE_LABPC
#  ifdef OS_LINUX   /* The LabPC+ driver is only useable under Linux. */

{"labpc_adc",      MXT_AIN_LABPC,     MXC_ANALOG_INPUT,   MXR_DEVICE,
				&mxd_labpc_adc_record_function_list,
				NULL,
				&mxd_labpc_adc_analog_input_function_list,
				&mxd_labpc_adc_num_record_fields,
				&mxd_labpc_adc_rfield_def_ptr},
{"labpc_dac",      MXT_AOU_LABPC,     MXC_ANALOG_OUTPUT,  MXR_DEVICE,
				&mxd_labpc_dac_record_function_list,
				NULL,
				&mxd_labpc_dac_analog_output_function_list,
				&mxd_labpc_dac_num_record_fields,
				&mxd_labpc_dac_rfield_def_ptr},
{"labpc_din",      MXT_DIN_LABPC,     MXC_DIGITAL_INPUT,  MXR_DEVICE,
				&mxd_labpc_din_record_function_list,
				NULL,
				&mxd_labpc_din_digital_input_function_list,
				&mxd_labpc_din_num_record_fields,
				&mxd_labpc_din_rfield_def_ptr},
{"labpc_dout",     MXT_DOU_LABPC,     MXC_DIGITAL_OUTPUT, MXR_DEVICE,
				&mxd_labpc_dout_record_function_list,
				NULL,
				&mxd_labpc_dout_digital_output_function_list,
				&mxd_labpc_dout_num_record_fields,
				&mxd_labpc_dout_rfield_def_ptr},
#  endif /* OS_LINUX */
#endif /* HAVE_LABPC */

{"am9513_motor",   MXT_MTR_AM9513,    MXC_MOTOR,         MXR_DEVICE,
				&mxd_am9513_motor_record_function_list,
				NULL,
				&mxd_am9513_motor_motor_function_list,
				&mxd_am9513_motor_num_record_fields,
				&mxd_am9513_motor_rfield_def_ptr},

{"am9513_scaler",  MXT_SCL_AM9513,    MXC_SCALER,         MXR_DEVICE,
				&mxd_am9513_scaler_record_function_list,
				NULL,
				&mxd_am9513_scaler_scaler_function_list,
				&mxd_am9513_scaler_num_record_fields,
				&mxd_am9513_scaler_rfield_def_ptr},

{"am9513_timer",   MXT_TIM_AM9513,    MXC_TIMER,          MXR_DEVICE,
				&mxd_am9513_timer_record_function_list,
				NULL,
				&mxd_am9513_timer_timer_function_list,
				&mxd_am9513_timer_num_record_fields,
				&mxd_am9513_timer_rfield_def_ptr},

#if HAVE_ORTEC_UMCBI

{"trump_mca",      MXT_MCA_TRUMP,     MXC_MULTICHANNEL_ANALYZER, MXR_DEVICE,
				&mxd_trump_record_function_list,
				NULL,
				&mxd_trump_mca_function_list,
				&mxd_trump_num_record_fields,
				&mxd_trump_rfield_def_ptr},

#endif /* HAVE_ORTEC_UMCBI */

#if HAVE_EPICS

{"epics_mca",      MXT_MCA_EPICS,     MXC_MULTICHANNEL_ANALYZER, MXR_DEVICE,
				&mxd_epics_mca_record_function_list,
				NULL,
				&mxd_epics_mca_mca_function_list,
				&mxd_epics_mca_num_record_fields,
				&mxd_epics_mca_rfield_def_ptr},

{"epics_mcs",      MXT_MCS_EPICS,     MXC_MULTICHANNEL_SCALER, MXR_DEVICE,
				&mxd_epics_mcs_record_function_list,
				NULL,
				&mxd_epics_mcs_mcs_function_list,
				&mxd_epics_mcs_num_record_fields,
				&mxd_epics_mcs_rfield_def_ptr},

#endif /* HAVE_EPICS */

  /* ===================== Scan types ===================== */

{"input_scan",     MXS_LIN_INPUT,     MXS_LINEAR_SCAN,    MXR_SCAN,
				&mxs_linear_scan_record_function_list,
				&mxs_linear_scan_scan_function_list,
				&mxs_input_linear_scan_function_list,
				&mxs_input_linear_scan_num_record_fields,
				&mxs_input_linear_scan_def_ptr},
{"motor_scan",     MXS_LIN_MOTOR,     MXS_LINEAR_SCAN,    MXR_SCAN,
				&mxs_linear_scan_record_function_list,
				&mxs_linear_scan_scan_function_list,
				&mxs_motor_linear_scan_function_list,
				&mxs_motor_linear_scan_num_record_fields,
				&mxs_motor_linear_scan_def_ptr},
{"2theta_scan",    MXS_LIN_2THETA,    MXS_LINEAR_SCAN,    MXR_SCAN,
				&mxs_linear_scan_record_function_list,
				&mxs_linear_scan_scan_function_list,
				&mxs_2theta_linear_scan_function_list,
				&mxs_2theta_linear_scan_num_record_fields,
				&mxs_2theta_linear_scan_def_ptr},
{"slit_scan",      MXS_LIN_SLIT,      MXS_LINEAR_SCAN,    MXR_SCAN,
				&mxs_linear_scan_record_function_list,
				&mxs_linear_scan_scan_function_list,
				&mxs_slit_linear_scan_function_list,
				&mxs_slit_linear_scan_num_record_fields,
				&mxs_slit_linear_scan_def_ptr},

{"file_list_scan", MXS_LST_FILE,      MXS_LIST_SCAN,      MXR_SCAN,
				&mxs_list_scan_record_function_list,
				&mxs_list_scan_scan_function_list,
				&mxs_file_list_scan_function_list,
				&mxs_file_list_scan_num_record_fields,
				&mxs_file_list_scan_def_ptr},

{"xafs_scan",      MXS_XAF_STANDARD,  MXS_XAFS_SCAN,      MXR_SCAN,
				&mxs_xafs_scan_record_function_list,
				&mxs_xafs_scan_scan_function_list,
				NULL,
				&mxs_xafs_std_scan_num_record_fields,
				&mxs_xafs_std_scan_def_ptr},

{"mcs_qscan",      MXS_QUI_MCS,       MXS_QUICK_SCAN,     MXR_SCAN,
				&mxs_mcs_quick_scan_record_function_list,
				&mxs_mcs_quick_scan_scan_function_list,
				NULL,
				&mxs_mcs_quick_scan_num_record_fields,
				&mxs_mcs_quick_scan_def_ptr},

#if HAVE_EPICS
{"joerger_qscan",  MXS_QUI_JOERGER,   MXS_QUICK_SCAN,     MXR_SCAN,
				&mxs_joerger_quick_scan_record_function_list,
				&mxs_joerger_quick_scan_scan_function_list,
				NULL,
				&mxs_joerger_quick_scan_num_record_fields,
				&mxs_joerger_quick_scan_def_ptr},

{"aps_id_qscan",   MXS_QUI_APS_ID,    MXS_QUICK_SCAN,     MXR_SCAN,
				&mxs_apsid_quick_scan_record_function_list,
				&mxs_apsid_quick_scan_scan_function_list,
				NULL,
				&mxs_apsid_quick_scan_num_record_fields,
				&mxs_apsid_quick_scan_def_ptr},

/* aps_id_qscan is a variant of the mcs_qscan scan type. */

#endif /* HAVE_EPICS */

  /* =================== Variable types ================== */

{"string",         MXV_INL_STRING,    MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_string_variable_num_record_fields,
				&mxv_inline_string_variable_def_ptr},
{"char",           MXV_INL_CHAR,      MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_char_variable_num_record_fields,
				&mxv_inline_char_variable_def_ptr},
{"uchar",          MXV_INL_UCHAR,     MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_uchar_variable_num_record_fields,
				&mxv_inline_uchar_variable_def_ptr},
{"short",          MXV_INL_SHORT,     MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_short_variable_num_record_fields,
				&mxv_inline_short_variable_def_ptr},
{"ushort",         MXV_INL_USHORT,    MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_ushort_variable_num_record_fields,
				&mxv_inline_ushort_variable_def_ptr},
{"int",            MXV_INL_INT,       MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_int_variable_num_record_fields,
				&mxv_inline_int_variable_def_ptr},
{"uint",           MXV_INL_UINT,      MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_uint_variable_num_record_fields,
				&mxv_inline_uint_variable_def_ptr},
{"long",           MXV_INL_LONG,      MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_long_variable_num_record_fields,
				&mxv_inline_long_variable_def_ptr},
{"ulong",          MXV_INL_ULONG,     MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_ulong_variable_num_record_fields,
				&mxv_inline_ulong_variable_def_ptr},
{"float",          MXV_INL_FLOAT,     MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_float_variable_num_record_fields,
				&mxv_inline_float_variable_def_ptr},
{"double",         MXV_INL_DOUBLE,    MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_double_variable_num_record_fields,
				&mxv_inline_double_variable_def_ptr},
{"record",         MXV_INL_RECORD,    MXV_INLINE,         MXR_VARIABLE,
				&mxv_inline_variable_record_function_list,
				NULL, NULL,
				&mxv_inline_record_variable_num_record_fields,
				&mxv_inline_record_variable_def_ptr},

{"net_string",     MXV_NET_STRING,    MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_string_variable_num_record_fields,
				&mxv_network_string_variable_dptr},
{"net_char",       MXV_NET_CHAR,      MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_char_variable_num_record_fields,
				&mxv_network_char_variable_dptr},
{"net_uchar",      MXV_NET_UCHAR,     MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_uchar_variable_num_record_fields,
				&mxv_network_uchar_variable_dptr},
{"net_short",      MXV_NET_SHORT,     MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_short_variable_num_record_fields,
				&mxv_network_short_variable_dptr},
{"net_ushort",     MXV_NET_USHORT,    MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_ushort_variable_num_record_fields,
				&mxv_network_ushort_variable_dptr},
{"net_int",        MXV_NET_INT,       MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_int_variable_num_record_fields,
				&mxv_network_int_variable_dptr},
{"net_uint",       MXV_NET_UINT,      MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_uint_variable_num_record_fields,
				&mxv_network_uint_variable_dptr},
{"net_long",       MXV_NET_LONG,      MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_long_variable_num_record_fields,
				&mxv_network_long_variable_dptr},
{"net_ulong",      MXV_NET_ULONG,     MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_ulong_variable_num_record_fields,
				&mxv_network_ulong_variable_dptr},
{"net_float",      MXV_NET_FLOAT,     MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_float_variable_num_record_fields,
				&mxv_network_float_variable_dptr},
{"net_double",     MXV_NET_DOUBLE,    MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				&mxv_network_variable_variable_function_list,
				NULL,
				&mxv_network_double_variable_num_record_fields,
				&mxv_network_double_variable_dptr},
{"net_record",     MXV_NET_RECORD,    MXV_NETWORK,         MXR_VARIABLE,
				&mxv_network_variable_record_function_list,
				NULL, NULL,
				&mxv_network_record_variable_num_record_fields,
				&mxv_network_record_variable_dptr},

#if HAVE_EPICS
{"epics_string",   MXV_EPI_STRING,    MXV_EPICS,         MXR_VARIABLE,
				&mxv_epics_variable_record_function_list,
				&mxv_epics_variable_variable_function_list,
				NULL,
				&mxv_epics_string_variable_num_record_fields,
				&mxv_epics_string_variable_def_ptr},
{"epics_char",     MXV_EPI_CHAR,      MXV_EPICS,         MXR_VARIABLE,
				&mxv_epics_variable_record_function_list,
				&mxv_epics_variable_variable_function_list,
				NULL,
				&mxv_epics_char_variable_num_record_fields,
				&mxv_epics_char_variable_def_ptr},
{"epics_short",    MXV_EPI_SHORT,     MXV_EPICS,         MXR_VARIABLE,
				&mxv_epics_variable_record_function_list,
				&mxv_epics_variable_variable_function_list,
				NULL,
				&mxv_epics_short_variable_num_record_fields,
				&mxv_epics_short_variable_def_ptr},
{"epics_long",     MXV_EPI_LONG,      MXV_EPICS,         MXR_VARIABLE,
				&mxv_epics_variable_record_function_list,
				&mxv_epics_variable_variable_function_list,
				NULL,
				&mxv_epics_long_variable_num_record_fields,
				&mxv_epics_long_variable_def_ptr},
{"epics_float",    MXV_EPI_FLOAT,     MXV_EPICS,         MXR_VARIABLE,
				&mxv_epics_variable_record_function_list,
				&mxv_epics_variable_variable_function_list,
				NULL,
				&mxv_epics_float_variable_num_record_fields,
				&mxv_epics_float_variable_def_ptr},
{"epics_double",   MXV_EPI_DOUBLE,    MXV_EPICS,         MXR_VARIABLE,
				&mxv_epics_variable_record_function_list,
				&mxv_epics_variable_variable_function_list,
				NULL,
				&mxv_epics_double_variable_num_record_fields,
				&mxv_epics_double_variable_def_ptr},

{"aps_topup_interlock", MXV_CAL_APS_TOPUP_INTERLOCK, MXV_CALC, MXR_VARIABLE,
				&mxv_aps_topup_record_function_list,
				&mxv_aps_topup_variable_function_list,
				NULL,
				&mxv_aps_topup_interlock_num_record_fields,
				&mxv_aps_topup_interlock_field_def_ptr},

{"aps_topup_time", MXV_CAL_APS_TOPUP_TIME_TO_INJECT, MXV_CALC, MXR_VARIABLE,
				&mxv_aps_topup_record_function_list,
				&mxv_aps_topup_variable_function_list,
				NULL,
				&mxv_aps_topup_time_to_inject_num_record_fields,
				&mxv_aps_topup_time_to_inject_field_def_ptr},

#endif /* HAVE_EPICS */

{"mathop",         MXV_CAL_MATHOP,    MXV_CALC,          MXR_VARIABLE,
				&mxv_mathop_record_function_list,
				&mxv_mathop_variable_function_list,
				NULL,
				&mxv_mathop_num_record_fields,
				&mxv_mathop_rfield_def_ptr},

{"old_mathop",     MXV_CAL_OLD_MATHOP,MXV_CALC,          MXR_VARIABLE,
				&mxv_old_mathop_record_function_list,
				&mxv_old_mathop_variable_function_list,
				NULL,
				&mxv_old_mathop_num_record_fields,
				&mxv_old_mathop_rfield_def_ptr},

  /* =================== Server types ================== */

#if USE_TCPIP
{"tcpip_server",   MXN_NET_TCPIP,     MXN_NETWORK_SERVER, MXR_SERVER,
				&mxn_tcpip_server_record_function_list,
				NULL, NULL,
				&mxn_tcpip_server_num_record_fields,
				&mxn_tcpip_server_rfield_def_ptr},
#endif

{"", 0, 0, 0, NULL, NULL, NULL, NULL, NULL}
};

