/*
 * Name:    v_old_mathop.h
 *
 * Purpose: Header file for simple mathematical operations on a list of records.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2001-2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef _V_OLD_MATHOP_H_
#define _V_OLD_MATHOP_H_

#define MXF_OLD_MATHOP_ADDITION		1
#define MXF_OLD_MATHOP_SUBTRACTION	2
#define MXF_OLD_MATHOP_MULTIPLICATION	3
#define MXF_OLD_MATHOP_DIVISION		4
#define MXF_OLD_MATHOP_NEGATION		5

#define MXF_OLD_MATHOP_SQUARE		11
#define MXF_OLD_MATHOP_SQUARE_ROOT	12

#define MXF_OLD_MATHOP_SINE		101
#define MXF_OLD_MATHOP_COSINE		102
#define MXF_OLD_MATHOP_TANGENT		103

#define MXF_OLD_MATHOP_ARC_SINE		111
#define MXF_OLD_MATHOP_ARC_COSINE	112
#define MXF_OLD_MATHOP_ARC_TANGENT	113

#define MXF_OLD_MATHOP_EXPONENTIAL	121
#define MXF_OLD_MATHOP_LOGARITHM	122
#define MXF_OLD_MATHOP_LOG10		123

typedef struct {
	MX_RECORD *destination_record;
	int operation_type;
	long num_records;
	char **record_name_array;

	MX_RECORD **record_array;
	double *value_array;

	int has_been_initialized;
} MX_OLD_MATHOP_VARIABLE;

#define MX_OLD_MATHOP_VARIABLE_STANDARD_FIELDS \
  {-1, -1, "destination_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_OLD_MATHOP_VARIABLE, destination_record),\
	{0}, NULL, MXFF_IN_DESCRIPTION }, \
  \
  {-1, -1, "operation_type", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_OLD_MATHOP_VARIABLE, operation_type), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY) }, \
  \
  {-1, -1, "num_records", MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_OLD_MATHOP_VARIABLE, num_records), \
	{0}, NULL, MXFF_IN_DESCRIPTION }, \
  \
  {-1, -1, "record_name_array", MXFT_STRING, \
			NULL, 2, {MXU_VARARGS_LENGTH, MXU_RECORD_NAME_LENGTH}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_OLD_MATHOP_VARIABLE, record_name_array), \
	{sizeof(char), sizeof(char *)}, NULL, \
		(MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY | MXFF_VARARGS ) }

MX_API_PRIVATE mx_status_type mxv_old_mathop_initialize_type( long );
MX_API_PRIVATE mx_status_type mxv_old_mathop_create_record_structures(
							MX_RECORD *record );

MX_API_PRIVATE mx_status_type mxv_old_mathop_finish_record_initialization(
							MX_RECORD *record );
MX_API_PRIVATE mx_status_type mxv_old_mathop_delete_record( MX_RECORD *record );
MX_API_PRIVATE mx_status_type mxv_old_mathop_dummy_function( MX_RECORD *record );

MX_API_PRIVATE mx_status_type mxv_old_mathop_send_variable( MX_VARIABLE *variable );
MX_API_PRIVATE mx_status_type mxv_old_mathop_receive_variable(
						MX_VARIABLE *variable );

extern MX_RECORD_FUNCTION_LIST mxv_old_mathop_record_function_list;
extern MX_VARIABLE_FUNCTION_LIST mxv_old_mathop_variable_function_list;

extern long mxv_old_mathop_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxv_old_mathop_rfield_def_ptr;

#endif /* _V_OLD_MATHOP_H_ */

