/*
 * Name:    ms_process_rs232.c
 *
 * Purpose: Functions used to process MX rs232 record events.
 *
 * Author:  William Lavender
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 2001-2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_driver.h"
#include "mx_record.h"
#include "mx_rs232.h"
#include "mx_socket.h"

#include "ms_server.h"
#include "ms_process_rs232.h"

mx_status_type
mxsrv_setup_rs232_process_functions( MX_RECORD *record )
{
	const char fname[] = "mxsrv_setup_rs232_process_functions()";

	MX_RECORD_FIELD *record_field;
	MX_RECORD_FIELD *record_field_array;
	long i;

	MX_DEBUG(2,("%s invoked.", fname));

	record_field_array = record->record_field_array;

	for ( i = 0; i < record->num_record_fields; i++ ) {

		record_field = &record_field_array[i];

		switch( record_field->label_value ) {
		case MXLV_232_GETCHAR:
		case MXLV_232_PUTCHAR:
		case MXLV_232_INPUT_IS_AVAILABLE:
		case MXLV_232_DISCARD_UNREAD_INPUT:
		case MXLV_232_DISCARD_UNWRITTEN_OUTPUT:
			record_field->process_function
					= mxsrv_rs232_process_function;
			break;
		default:
			break;
		}
	}
	return MX_SUCCESSFUL_RESULT;
}

mx_status_type
mxsrv_rs232_process_function( void *record_ptr,
			void *record_field_ptr, int operation )
{
	const char fname[] = "mxsrv_rs232_process_function()";

	MX_RECORD *record;
	MX_RECORD_FIELD *record_field;
	MX_RS232 *rs232;
	char char_value;
	int int_value;
	mx_status_type status;

	record = (MX_RECORD *) record_ptr;
	record_field = (MX_RECORD_FIELD *) record_field_ptr;
	rs232 = (MX_RS232 *) (record->record_class_struct);

	status = MX_SUCCESSFUL_RESULT;

	switch( operation ) {
	case MX_PROCESS_GET:
		switch( record_field->label_value ) {
		case MXLV_232_GETCHAR:
			status = mx_rs232_getchar( record, &char_value, 0 );

			rs232->getchar_value = (int) char_value;
			break;
		case MXLV_232_INPUT_IS_AVAILABLE:
			status = mx_rs232_input_is_available( record,
								&int_value );

			rs232->input_is_available = int_value;
			break;
		default:
			MX_DEBUG( 1,(
			    "%s: *** Unknown MX_PROCESS_GET label value = %ld",
				fname, record_field->label_value));
			break;
		}
		break;
	case MX_PROCESS_PUT:
		switch( record_field->label_value ) {
		case MXLV_232_PUTCHAR:
			char_value = (char) rs232->putchar_value;

			status = mx_rs232_putchar( record, char_value, 0 );
			break;
		case MXLV_232_DISCARD_UNREAD_INPUT:
			status = mx_rs232_discard_unread_input( record, 0 );
			break;
		case MXLV_232_DISCARD_UNWRITTEN_OUTPUT:
			status = mx_rs232_discard_unwritten_output( record, 0 );
			break;
		default:
			MX_DEBUG( 1,(
			    "%s: *** Unknown MX_PROCESS_PUT label value = %ld",
				fname, record_field->label_value));
			break;
		}
		break;
	default:
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Unknown operation code = %d", operation );
		break;
	}

	return status;
}

