/*
 * Name:    ms_server.h
 *
 * Purpose: Header file for MX server related structures and functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 1999-2001 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __MS_SERVER_H__
#define __MS_SERVER_H__

#include "mx_net.h"

#define MXF_SRV_MX_SERVER_EVENT_TYPE	1
#define MXF_SRV_MX_CLIENT_EVENT_TYPE	2
#define MXF_SRV_EPICS_EVENT_TYPE	3

typedef struct {
	MX_SOCKET synchronous_socket;
	MX_SOCKET callback_socket;
	int handler_array_index;
	struct mxsrv_event_handler_type *event_handler;
	MX_NETWORK_MESSAGE_BUFFER *message_buffer;
} MXSRV_SOCKET_HANDLER;

typedef struct {
	int max_sockets;
	int num_sockets_in_use;
	int handler_array_size;
	MXSRV_SOCKET_HANDLER **array;
} MXSRV_SOCKET_HANDLER_LIST;

struct mxsrv_queued_event_type {
	MXSRV_SOCKET_HANDLER *socket_handler;
	MX_RECORD *record;
	MX_RECORD_FIELD *record_field;
	void *event_data;
	struct mxsrv_queued_event_type *next_event;
};

typedef struct mxsrv_queued_event_type MXSRV_QUEUED_EVENT;

struct mxsrv_event_handler_type {
	long type;
	char name[MXU_RECORD_NAME_LENGTH + 1];
	void *type_struct;

	mx_status_type ( *init )(MX_RECORD *record_list,
			MXSRV_SOCKET_HANDLER_LIST *socket_handler_list,
			struct mxsrv_event_handler_type *event_handler);

	mx_status_type ( *process_event )(MX_RECORD *record_list,
			MXSRV_SOCKET_HANDLER *socket_handler,
			MXSRV_SOCKET_HANDLER_LIST *socket_handler_list,
			struct mxsrv_event_handler_type *event_handler);

	mx_status_type ( *process_queued_event )(
			MXSRV_QUEUED_EVENT *queued_event);
};

typedef struct mxsrv_event_handler_type MXSRV_EVENT_HANDLER;

extern MXSRV_EVENT_HANDLER mxsrv_event_handler_list[];
extern int mxsrv_num_event_handlers;

/*---*/

/* An MX process function is invoked when an MX_RECORD_FIELD is read or
 * written, unles the pointer to the function in the MX_RECORD_FIELD array
 * is NULL, or the pointer to the MX_RECORD_FIELD array is NULL.
 *
 * MX process functions are declared as follows:
 *
 * extern mx_status_type ( *mx_process_function )
 *	( void *record,			! MX_RECORD ptr
 *	  void *record_field,		! MX_RECORD_FIELD ptr
 *	  int operation );
 *
 * I would like to make the above a typedef, but there doesn't seem to be
 * a way to usefully make use of such a typedef, so all we're left with
 * is this comment.
 */

#define MX_PROCESS_GET    1
#define MX_PROCESS_PUT    2

/*---*/

struct mxsrv_callback_type {
	mx_status_type ( *callback_function ) ( MX_RECORD_FIELD *record_field,
						void *callback_argument );
	void *callback_argument;
	int callback_enabled;
	struct mxsrv_callback_type *next_callback;
};

typedef struct mxsrv_callback_type MXSRV_CALLBACK;

typedef struct {
	MXSRV_CALLBACK *value_changed_callback_list;
} MXSRV_CALLBACK_LIST;

#define MXSF_VALUE_CHANGED_CALLBACK	1

extern mx_status_type mxsrv_add_queued_event(
				MXSRV_SOCKET_HANDLER *socket_handler,
				MX_RECORD *record,
				MX_RECORD_FIELD *record_field,
				void *event_data );

extern mx_status_type mxsrv_process_queued_event(
				MX_RECORD *record,
				MX_CLOCK_TICK current_clock_time );

extern mx_status_type mxsrv_add_callback(
				long callback_type,
				MX_RECORD_FIELD *field,
				mx_status_type ( *callback_function )
						( MX_RECORD_FIELD *, void *),
				void *callback_argument );

extern mx_status_type mxsrv_delete_callback(
				long callback_type,
				MX_RECORD_FIELD *field,
				mx_status_type ( *callback_function )
						( MX_RECORD_FIELD *, void *),
				void *callback_argument );

extern mx_status_type mxsrv_find_callback(
				long callback_type,
				MX_RECORD_FIELD *field,
				mx_status_type ( *callback_function )
						( MX_RECORD_FIELD *, void *),
				void *callback_argument,
				MXSRV_CALLBACK **callback_found );

extern mx_status_type mxsrv_invoke_callback_list(
				long callback_type,
				MX_RECORD_FIELD *field );

/*---*/

#endif /* __MS_SERVER_H__ */

